@@ -1,18 +1,20 @@
 /*
-  Copyright (c) 2013-2016 Montel Laurent <montel@kde.org>
+   Copyright (C) 2013-2018 Laurent Montel <montel@kde.org>
 
-  This program is free software; you can redistribute it and/or modify it
-  under the terms of the GNU General Public License, version 2, as
-  published by the Free Software Foundation.
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
 
-  This program is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  General Public License for more details.
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
 
-  You should have received a copy of the GNU General Public License along
-  with this program; if not, write to the Free Software Foundation, Inc.,
-  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "sieveactionwidgetlister.h"
@@ -23,7 +25,7 @@
 #include "sieveactions/sieveactionlist.h"
 #include "commonwidgets/sievehelpbutton.h"
 #include "autocreatescriptutil_p.h"
-#include "PimCommon/MinimumComboBox"
+#include <PimCommon/MinimumComboBox>
 
 #include <QPushButton>
 #include <KLocalizedString>
@@ -34,23 +36,17 @@
 #include "libksieve_debug.h"
 #include <QToolButton>
 #include <QWhatsThis>
-#include <QDomElement>
+
+#include <autocreatescripts/sieveactions/sieveactionsetvariable.h>
 
 using namespace KSieveUi;
 
 static int MINIMUMACTION = 1;
 static int MAXIMUMACTION = 8;
 
-namespace
-{
-inline const QString indentation()
-{
-    return QStringLiteral("    ");
-}
-}
-
-SieveActionWidget::SieveActionWidget(QWidget *parent)
+SieveActionWidget::SieveActionWidget(SieveEditorGraphicalModeWidget *graphicalModeWidget, QWidget *parent)
     : QWidget(parent)
+    , mSieveGraphicalModeWidget(graphicalModeWidget)
 {
     initWidget();
 }
@@ -63,7 +59,7 @@ SieveActionWidget::~SieveActionWidget()
 
 bool SieveActionWidget::isConfigurated() const
 {
-    return (mComboBox->currentIndex() != (mComboBox->count() - 1));
+    return mComboBox->currentIndex() != (mComboBox->count() - 1);
 }
 
 void SieveActionWidget::setFilterAction(QWidget *widget)
@@ -79,23 +75,37 @@ void SieveActionWidget::setFilterAction(QWidget *widget)
     }
 }
 
-void SieveActionWidget::generatedScript(QString &script, QStringList &requires, bool onlyActions)
+void SieveActionWidget::generatedScript(QString &script, QStringList &required, bool onlyActions, bool inForEveryPartLoop)
 {
     const int index = mComboBox->currentIndex();
     if (index != mComboBox->count() - 1) {
         KSieveUi::SieveAction *widgetAction = mActionList.at(mComboBox->currentIndex());
         QWidget *currentWidget = mLayout->itemAtPosition(1, 3)->widget();
         const QStringList lstRequires = widgetAction->needRequires(currentWidget);
-        Q_FOREACH (const QString &r, lstRequires) {
-            if (!requires.contains(r)) {
-                requires.append(r);
+        for (const QString &r : lstRequires) {
+            if (!required.contains(r)) {
+                required.append(r);
             }
         }
         QString comment = widgetAction->comment();
-        if (!comment.isEmpty()) {
-            script += (onlyActions ? QString() : indentation()) + QLatin1Char('#') + comment.replace(QLatin1Char('\n'), QStringLiteral("\n#")) + QLatin1Char('\n');
+        QString indent;
+        if (!onlyActions) {
+            indent += AutoCreateScriptUtil::indentation();
         }
-        script += (onlyActions ? QString() : indentation()) + widgetAction->code(currentWidget) + QLatin1Char('\n');
+        if (inForEveryPartLoop) {
+            indent += AutoCreateScriptUtil::indentation();
+        }
+        if (!comment.trimmed().isEmpty()) {
+            const QVector<QStringRef> commentList = comment.splitRef(QLatin1Char('\n'));
+            for (const QStringRef &str : commentList) {
+                if (str.isEmpty()) {
+                    script += QLatin1Char('\n');
+                } else {
+                    script += indent + QLatin1Char('#') + str + QLatin1Char('\n');
+                }
+            }
+        }
+        script += indent + widgetAction->code(currentWidget) + QLatin1Char('\n');
     }
 }
 
@@ -106,13 +116,13 @@ void SieveActionWidget::initWidget()
 
     mComboBox = new PimCommon::MinimumComboBox;
     mComboBox->setEditable(false);
-    const QList<KSieveUi::SieveAction *> list = KSieveUi::SieveActionList::actionList();
+    const QList<KSieveUi::SieveAction *> list = KSieveUi::SieveActionList::actionList(mSieveGraphicalModeWidget);
     QList<KSieveUi::SieveAction *>::const_iterator it;
     QList<KSieveUi::SieveAction *>::const_iterator end(list.constEnd());
     int index = 0;
-    QStringList listCapabilities = SieveEditorGraphicalModeWidget::sieveCapabilities();
+    QStringList listCapabilities = mSieveGraphicalModeWidget->sieveCapabilities();
     //imapflags was old name of imap4flags but still used.
-    if (listCapabilities.contains(QStringLiteral("imap4flags"))) {
+    if (listCapabilities.contains(QLatin1String("imap4flags"))) {
         listCapabilities.append(QStringLiteral("imapflags"));
     }
     for (index = 0, it = list.constBegin(); it != end; ++it, ++index) {
@@ -135,20 +145,17 @@ void SieveActionWidget::initWidget()
         }
     }
 
-    mHelpButton = new SieveHelpButton;
-    mHelpButton->setEnabled(false);
+    mHelpButton = new SieveHelpButton(this);
     connect(mHelpButton, &SieveHelpButton::clicked, this, &SieveActionWidget::slotHelp);
     mLayout->addWidget(mHelpButton, 1, 0);
 
-    mCommentButton = new QToolButton;
+    mCommentButton = new QToolButton(this);
     mCommentButton->setToolTip(i18n("Add comment"));
-    mCommentButton->setEnabled(false);
     mLayout->addWidget(mCommentButton, 1, 1);
     mCommentButton->setIcon(QIcon::fromTheme(QStringLiteral("view-pim-notes")));
     connect(mCommentButton, &QToolButton::clicked, this, &SieveActionWidget::slotAddComment);
 
     mComboBox->addItem(QLatin1String(""));
-    mComboBox->setCurrentIndex(mComboBox->count() - 1);
 
     mComboBox->setMaxCount(mComboBox->count());
     mComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed));
@@ -158,7 +165,7 @@ void SieveActionWidget::initWidget()
 
     updateGeometry();
 
-    connect(mComboBox, static_cast<void (PimCommon::MinimumComboBox::*)(int)>(&PimCommon::MinimumComboBox::activated), this, &SieveActionWidget::slotActionChanged);
+    connect(mComboBox, QOverload<int>::of(&PimCommon::MinimumComboBox::activated), this, &SieveActionWidget::slotActionChanged);
 
     mAdd = new QPushButton(this);
     mAdd->setIcon(QIcon::fromTheme(QStringLiteral("list-add")));
@@ -175,7 +182,8 @@ void SieveActionWidget::initWidget()
 
     connect(mAdd, &QPushButton::clicked, this, &SieveActionWidget::slotAddWidget);
     connect(mRemove, &QPushButton::clicked, this, &SieveActionWidget::slotRemoveWidget);
-    setFilterAction(Q_NULLPTR);
+
+    clear();
 }
 
 void SieveActionWidget::slotHelp()
@@ -190,6 +198,14 @@ void SieveActionWidget::slotHelp()
     }
 }
 
+void SieveActionWidget::clear()
+{
+    mComboBox->setCurrentIndex(mComboBox->count() - 1);
+    setFilterAction(nullptr);
+    mCommentButton->setEnabled(false);
+    mHelpButton->setEnabled(false);
+}
+
 void SieveActionWidget::slotAddComment()
 {
     const int index = mComboBox->currentIndex();
@@ -219,7 +235,7 @@ void SieveActionWidget::slotActionChanged(int index)
     } else {
         mAdd->setEnabled(true);
         mCommentButton->setEnabled(false);
-        setFilterAction(Q_NULLPTR);
+        setFilterAction(nullptr);
         mHelpButton->setEnabled(false);
     }
     Q_EMIT valueChanged();
@@ -243,7 +259,23 @@ void SieveActionWidget::updateAddRemoveButton(bool addButtonEnabled, bool remove
     mRemove->setEnabled(removeButtonEnabled);
 }
 
-bool SieveActionWidget::setAction(const QString &actionName, const QDomElement &element, const QString &comment, QString &error)
+void SieveActionWidget::setLocaleVariable(const SieveGlobalVariableActionWidget::VariableElement &var)
+{
+    const int index = mComboBox->findData(QStringLiteral("set"));
+    if (index != -1) {
+        mComboBox->setCurrentIndex(index);
+        slotActionChanged(index);
+        KSieveUi::SieveActionSetVariable *localVar = dynamic_cast<KSieveUi::SieveActionSetVariable *>(mActionList.at(index));
+        if (localVar) {
+            localVar->setLocalVariable(this, var);
+        }
+    } else {
+        //error += i18n("Script contains unsupported feature \"%1\"", actionName) + QLatin1Char('\n');
+        //qCDebug(LIBKSIEVE_LOG) << "Action " << actionName << " not supported";
+    }
+}
+
+bool SieveActionWidget::setAction(const QString &actionName, QXmlStreamReader &element, const QString &comment, QString &error)
 {
     const int index = mComboBox->findData(actionName);
     bool result = false;
@@ -253,12 +285,17 @@ bool SieveActionWidget::setAction(const QString &actionName, const QDomElement &
         KSieveUi::SieveAction *action = mActionList.at(index);
         result = action->setParamWidgetValue(element, this, error);
         action->setComment(comment);
+    } else {
+        error += i18n("Script contains unsupported feature \"%1\"", actionName) + QLatin1Char('\n');
+        qCDebug(LIBKSIEVE_LOG) << "Action " << actionName << " not supported";
+        element.skipCurrentElement();
     }
     return result;
 }
 
-SieveActionWidgetLister::SieveActionWidgetLister(QWidget *parent)
+SieveActionWidgetLister::SieveActionWidgetLister(SieveEditorGraphicalModeWidget *graphicalModeWidget, QWidget *parent)
     : KPIM::KWidgetLister(false, MINIMUMACTION, MAXIMUMACTION, parent)
+    , mSieveGraphicalModeWidget(graphicalModeWidget)
 {
     slotClear();
     updateAddRemoveButton();
@@ -266,7 +303,6 @@ SieveActionWidgetLister::SieveActionWidgetLister(QWidget *parent)
 
 SieveActionWidgetLister::~SieveActionWidgetLister()
 {
-
 }
 
 void SieveActionWidgetLister::slotAddWidget(QWidget *w)
@@ -305,14 +341,14 @@ void SieveActionWidgetLister::updateAddRemoveButton()
     }
 }
 
-void SieveActionWidgetLister::generatedScript(QString &script, QStringList &requires, bool onlyActions)
+void SieveActionWidgetLister::generatedScript(QString &script, QStringList &requireModules, bool onlyActions, bool inForEveryPartLoop)
 {
     const QList<QWidget *> widgetList = widgets();
     QList<QWidget *>::ConstIterator wIt = widgetList.constBegin();
     QList<QWidget *>::ConstIterator wEnd = widgetList.constEnd();
     for (; wIt != wEnd; ++wIt) {
         SieveActionWidget *w = qobject_cast<SieveActionWidget *>(*wIt);
-        w->generatedScript(script, requires, onlyActions);
+        w->generatedScript(script, requireModules, onlyActions, inForEveryPartLoop);
     }
 }
 
@@ -325,14 +361,16 @@ void SieveActionWidgetLister::reconnectWidget(SieveActionWidget *w)
 
 void SieveActionWidgetLister::clearWidget(QWidget *aWidget)
 {
-    Q_UNUSED(aWidget);
+    if (aWidget) {
+        SieveActionWidget *widget = static_cast<SieveActionWidget *>(aWidget);
+        widget->clear();
+    }
     Q_EMIT valueChanged();
-    //TODO
 }
 
 QWidget *SieveActionWidgetLister::createWidget(QWidget *parent)
 {
-    SieveActionWidget *w = new SieveActionWidget(parent);
+    SieveActionWidget *w = new SieveActionWidget(mSieveGraphicalModeWidget, parent);
     reconnectWidget(w);
     return w;
 }
@@ -342,67 +380,79 @@ int SieveActionWidgetLister::actionNumber() const
     return widgets().count();
 }
 
-void SieveActionWidgetLister::loadScript(const QDomElement &element, bool onlyActions, QString &error)
+void SieveActionWidgetLister::loadLocalVariable(const SieveGlobalVariableActionWidget::VariableElement &var)
+{
+    SieveActionWidget *w = qobject_cast<SieveActionWidget *>(widgets().constLast());
+    if (w->isConfigurated()) {
+        addWidgetAfterThisWidget(widgets().constLast());
+        w = qobject_cast<SieveActionWidget *>(widgets().constLast());
+    }
+    w->setLocaleVariable(var);
+}
+
+void SieveActionWidgetLister::loadScript(QXmlStreamReader &element, bool onlyActions, QString &error)
 {
     QString comment;
     if (onlyActions) {
-        if (!element.isNull()) {
-            const QString tagName = element.tagName();
-            if (tagName == QLatin1String("action")) {
-                if (element.hasAttribute(QStringLiteral("name"))) {
-                    const QString actionName = element.attribute(QStringLiteral("name"));
-                    SieveActionWidget *w = qobject_cast<SieveActionWidget *>(widgets().last());
-                    if (w->isConfigurated()) {
-                        addWidgetAfterThisWidget(widgets().last());
-                        w = qobject_cast<SieveActionWidget *>(widgets().last());
-                    }
-                    w->setAction(actionName, element, comment, error);
-                    //comment.clear();
-                } else if (tagName == QLatin1String("crlf")) {
-                    //nothing
-                } else {
-                    qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript don't have name attribute " << tagName;
+        const QStringRef tagName = element.name();
+        if (tagName == QLatin1String("action")) {
+            if (element.attributes().hasAttribute(QLatin1String("name"))) {
+                const QString actionName = element.attributes().value(QLatin1String("name")).toString();
+                SieveActionWidget *w = qobject_cast<SieveActionWidget *>(widgets().constLast());
+                if (w->isConfigurated()) {
+                    addWidgetAfterThisWidget(widgets().constLast());
+                    w = qobject_cast<SieveActionWidget *>(widgets().constLast());
                 }
+                w->setAction(actionName, element, comment, error);
+                //comment.clear();
+            } else if (tagName == QLatin1String("crlf")) {
+                element.skipCurrentElement();
+                //nothing
+            } else {
+                qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript don't have name attribute " << tagName;
             }
+        } else {
+            qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript Unknow tag name " << tagName;
         }
     } else {
         bool firstAction = true;
-        QDomNode node = element.firstChild();
-        while (!node.isNull()) {
-            QDomElement e = node.toElement();
-            if (!e.isNull()) {
-                const QString tagName = e.tagName();
-                if (tagName == QLatin1String("action") || tagName == QLatin1String("control")/*for break action*/) {
-                    if (e.hasAttribute(QStringLiteral("name"))) {
-                        const QString actionName = e.attribute(QStringLiteral("name"));
-                        if (tagName == QLatin1String("control") && actionName == QLatin1String("if")) {
-                            qCDebug(LIBKSIEVE_LOG) << "We found an loop if in a loop if. Not supported";
-                            error += QLatin1Char('\n') + i18n("We detected a loop if in a loop if. It's not supported") + QLatin1Char('\n');
-                        }
-                        if (firstAction) {
-                            firstAction = false;
-                        } else {
-                            addWidgetAfterThisWidget(widgets().last());
-                        }
-                        SieveActionWidget *w = qobject_cast<SieveActionWidget *>(widgets().last());
-                        w->setAction(actionName, e, comment, error);
-                        comment.clear();
+        bool previousActionWasAComment = false;
+        while (element.readNextStartElement()) {
+            const QStringRef tagName = element.name();
+            if (tagName == QLatin1String("action") || tagName == QLatin1String("control") /*for break action*/) {
+                if (element.attributes().hasAttribute(QLatin1String("name"))) {
+                    const QString actionName = element.attributes().value(QLatin1String("name")).toString();
+                    if (tagName == QLatin1String("control") && actionName == QLatin1String("if")) {
+                        qCDebug(LIBKSIEVE_LOG) << "We found an loop if in a loop if. Not supported";
+                        error += i18n("We detected a loop if in a loop if. It's not supported") + QLatin1Char('\n');
+                    }
+                    if (firstAction) {
+                        firstAction = false;
                     } else {
-                        qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript don't have name attribute " << tagName;
+                        addWidgetAfterThisWidget(widgets().constLast());
                     }
-                } else if (tagName == QLatin1String("comment")) {
-                    if (!comment.isEmpty()) {
-                        comment += QLatin1Char('\n');
-                    }
-                    comment += e.text();
-                } else if (tagName == QLatin1String("crlf")) {
-                    //nothing
+                    SieveActionWidget *w = qobject_cast<SieveActionWidget *>(widgets().constLast());
+                    w->setAction(actionName, element, comment, error);
+                    comment.clear();
                 } else {
-                    qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript unknown tagName " << tagName;
+                    qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript don't have name attribute " << tagName;
                 }
+                previousActionWasAComment = false;
+            } else if (tagName == QLatin1String("comment")) {
+                if (!comment.isEmpty()) {
+                    comment += QLatin1Char('\n');
+                }
+                previousActionWasAComment = true;
+                comment += element.readElementText();
+            } else if (tagName == QLatin1String("crlf")) {
+                //Add new line if previous action was a comment
+                if (previousActionWasAComment) {
+                    comment += QLatin1Char('\n');
+                }
+                element.skipCurrentElement();
+            } else {
+                qCDebug(LIBKSIEVE_LOG) << " SieveActionWidgetLister::loadScript unknown tagName " << tagName;
             }
-            node = node.nextSibling();
         }
     }
 }
-
